#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008 All Rights Reserved 

package cqcli::commonservice;
use warnings;
require Exporter;
use 5.008006;
use Error qw(:try);
use Config;
use cqcli::Exception;
use cqcli::Util;


@ISA = qw(Exporter);
$VERSION = '0.01';
%EXPORT_TAGS = ( 'all' => [ qw() ] );
@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
@EXPORT = qw(InitCqcliCommonService   GetRecordTypeInfoObj GetRecordInfoObj
		GetSession CreateSession DestroySession GetCommandMode GetPromptCount GetJavaHash
		GetLoginCredentials GetSchemaName GetDBName GetPassword GetUserName GetActionName
		IsAskMode IsAPIMode IsInteractiveMode SetCQCLIAskMode SetCQCLIAPIMode SetCQCLIInteractiveMode
		GetAllFieldsInterActively GetFieldValueInterActively GetRecordId GetDbId 
		ValidateFieldValue GetJavaPayload
		SetRequestObject SetResponseObject SetSessionScope IsCommandScopeSession GetResponseObject 
		GetRequestObject CheckPromptModeValidity CheckPromptMode getDefaultRecordtype
		);

BEGIN {
	my $pkgdir = undef;
	my $archinlinedir = undef;
	my $inlinedir = undef;
	$pkgdir = __FILE__;
	$pkgdir =~ s/\/commonservice\.pm//i;
	$pkgdir =~ s/\\commonservice\.pm//i;
	if($Config{osname} eq "MSWin32"){
		$pkgdir =~ s/\//\\/g;
		$archinlinedir = "inlineclass"."\\".$Config{archname};
		$inlinedir = $pkgdir."\\".$archinlinedir;
	}else{
		$pkgdir =~ s/\\/\//g;
		$archinlinedir = "inlineclass"."/".$Config{archname};
		$inlinedir = $pkgdir."/".$archinlinedir;
	}
	if(!(-d $inlinedir)){
		die "$! $inlinedir : please create the directory manually\n"
	}
	$ENV{PERL_INLINE_JAVA_DIRECTORY} = $inlinedir;
}
################# PerlJava Factory ##########################

package cqcli::PerlInlineJava;



use Inline (

Java => <<'END_OF_JAVA_CODE' ,
import com.ibm.rational.CQTool.Session.CQSessionManager;
import com.ibm.rational.CQTool.Session.CQSession;
import com.ibm.rational.CQTool.Session.ICLISession;
import com.ibm.rational.CQTool.Utility.CQCLIJavaUtility;
import com.ibm.rational.CQTool.RecordTypeInfo;
import com.ibm.rational.CQTool.RecordInfo;
import com.ibm.rational.Payload.Payload;
import java.util.HashMap;
import java.util.Map;
import java.lang.String;

public class PerlInlineJavaFactory {
	
	private static CQSessionManager sessionManager = null;
	public static RecordInfo ObjRInfo = null;

	public PerlInlineJavaFactory() 
	{

	}
	public static CQSessionManager createSessionManager() 
	{
		if(PerlInlineJavaFactory.sessionManager==null){
			PerlInlineJavaFactory.sessionManager = CQSessionManager.createInstance();
		}
		return sessionManager;
	}	
	
	public static void DestroySessionManager() {
		PerlInlineJavaFactory.sessionManager = null;
		
	}

	public static RecordTypeInfo createRecordTypeInfo(String RecordType) throws java.lang.Exception
	{
		if( RecordType == null || RecordType.length()==0)
			RecordType = "Defect";
		return RecordTypeInfo.createInstance(RecordType);
			
	}
	
	public static RecordInfo createRecordInfo() throws java.lang.Exception
	{
		if(PerlInlineJavaFactory.ObjRInfo == null)
			PerlInlineJavaFactory.ObjRInfo = RecordInfo.createInstance();
		
		return PerlInlineJavaFactory.ObjRInfo;
			
	}
	
	public static HashMap createJavaHashMap(){
        HashMap h = new HashMap();
         return h ;
    }

	public static Payload createJavaPayload(String cmdname, HashMap requestObj) {
		Payload payloadObj = new Payload(cmdname,requestObj);
		return payloadObj;
	}

}

END_OF_JAVA_CODE
AUTOSTUDY => 1,
DIRECTORY => $ENV{PERL_INLINE_JAVA_DIRECTORY},

);

#####################CQ Session Manager###############################


package cqcli::SessionManager;
use cqcli::Util;


use Inline::Java qw(caught cast) ;
sub new 
{
	my ($class , @args) = @_;
	my $objsm = {
		'session'	=> undef,
		'sessionm'  => undef,
	};
	eval {
		$objsm->{'sessionm'} =  cqcli::PerlInlineJava::PerlInlineJavaFactory->createSessionManager();
	};
	if ($@)
	{
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_Failure_SM");
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}
	bless $objsm , ref($class)||$class;
	return $objsm;
}

sub CreateSession
{
	my ($self , %cred) = @_;
	my $psession = undef;

	eval {
		#$self->{'session'}=$self->{'session'}->createCQSession($cred{'login'} ,$cred{'password'},$cred{'connection'},$cred{'db'} );
		#$self->{'session'} = $self->{'sessionm'}->createCQSession("admin", "admin","7.0.0","SAMPL");
		$self->{'session'} = $self->{'sessionm'}->createCQSession($cred{'user'}, $cred{'password'},$cred{'schema'},$cred{'database'});
	};
	if ($@)
	{
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_Failure_Login");
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}
	return $self->{'session'};
}


sub DestroySession
{
	my ($self , %cred) = @_;
	my $session = undef;

	eval {
		$session = $self->{'sessionm'}->destroySession();
	};
	if ($@){
		return 0;
	}
	return 0;
}


######################## CQ RecordtypeInfo  #################################

package cqcli::RecordTypeInfo;

use cqcli::Util;
use Inline::Java qw(caught cast);


%EXPORT_TAGS = ( 'all' => [ qw() ] );
@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
@EXPORT = qw();


sub new
{
	my ($caller , $recordtype) = @_;
	my $id = 1;
	
	my $self = {'obj' =>undef,
				'type' => 'Defect'};
	eval { 
		$self->{'type'} = $recordtype if (defined $recordtype);
		$self->{'obj'} = cqcli::PerlInlineJava::PerlInlineJavaFactory->createRecordTypeInfo($self->{'type'});
		$self->{'obj'}->setUpFieldReferences($id);
	};

	if ($@)
	{
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_Err_RecordType",$self->{'type'});
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}
	bless $self,ref($caller)||$caller;

	return $self->{'obj'};
}


##########################CQ Record Info #############################


package cqcli::RecordInfo;

use cqcli::Util;
use Inline::Java qw(caught cast);

sub new
{
	my ($caller , $recordtype) = @_;

	my $self = { };
	eval { 
		$self->{'obj'} = cqcli::PerlInlineJava::PerlInlineJavaFactory->createRecordInfo(); 
	};
	if ($@)
	{
		if (caught("java.lang.Exception"))	{
			ThrowJavaException( $@,"CQCLI_Err_RecordInfo");
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}
	bless $self,ref($caller)||$caller;

	return $self->{'obj'};
}


###############################################################
package cqcli::commonservice;
use Inline::Java qw(caught cast);
use cqcli::IOHandler;

our $CLICmdMode = { 'api' => undef,
				 'ask' => undef,
				 'interactive' => undef,
				 'promptmode'    => undef,
};

our $RESOURCECACHE = {
	'controller'	=> undef ,
	'session'		=> undef ,
	'smanager'		=> undef ,
	'rinfo'			=> undef,
	'rtypeinfo'		=> { },
};

sub InitCqcliCommonService()
{
	InitIOHandler();
	InitSessionManager();
	#InitRecordObj();
	
}

sub GetRecordTypeInfoObj
{
	my $rtype = shift; 
	$RESOURCECACHE->{'rtypeinfo'}->{$rtype} = new cqcli::RecordTypeInfo($rtype) 
		unless defined $RESOURCECACHE->{'rtypeinfo'}->{$rtype}; 
	return $RESOURCECACHE->{'rtypeinfo'}->{$rtype};
}

sub GetRecordInfoObj
{
	$RESOURCECACHE->{'rinfo'} = new cqcli::RecordInfo() 
				unless defined $RESOURCECACHE->{'rinfo'}; 
	return $RESOURCECACHE->{'rinfo'};
}

sub CreateSession
{
	my %session = @_;

	InitSessionManager()
		unless defined $RESOURCECACHE->{'sessionm'};

	$RESOURCECACHE->{'session'} = $RESOURCECACHE->{'sessionm'}->CreateSession(%session);
	return $RESOURCECACHE->{'session'};
}

sub DestroySession()
{
	InitSessionManager()
		unless defined $RESOURCECACHE->{'sessionm'};
	$RESOURCECACHE->{'sessionm'}->DestroySession();
	$RESOURCECACHE->{'session'}=undef;
}


sub GetSession
{
	if ($RESOURCECACHE->{'session'}) {
		return $RESOURCECACHE->{'session'};
	}
	return undef;
}

sub GetJavaHash()
{
	my $hash = undef;
	eval { 
		$hash = cqcli::PerlInlineJava::PerlInlineJavaFactory->createJavaHashMap(); 
	};
	if ($@)
	{
		if (caught("java.lang.Exception"))	{
			ThrowJavaException( $@,"CQCLI_Err_JavaHash");
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}
	return $hash;
}

sub GetJavaPayload
{
	my ($cmd , $hashmap) = @_;
	my $payloadhash = undef;
	eval { 
		$payloadhash = cqcli::PerlInlineJava::PerlInlineJavaFactory->createJavaPayload($cmd , $hashmap); 
	};
	if ($@)
	{
		if (caught("java.lang.Exception"))	{
			ThrowJavaException( $@,"CQCLI_Err_JavaHash");
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}
	return $payloadhash;
}

sub getDefaultRecordtype
{
  my $session = GetSession();
  return $session->getDefaultRecordType();	
}

sub ValidateFieldValue
{
	my ($fname,$fvalue,$rtype,$recid,$idflag)=@_;
	my ($recordtypeobj,$isChiceList,$val,$choiceList)=(undef,undef,undef,undef);
	my @Clist = ();

	try 
	{
		$recordtypeobj = GetRecordTypeInfoObj($rtype);
		$isChiceList = $recordtypeobj->isChoiceList($fname,$recid,$idflag);

		if(!($isChiceList)){
			return 1;
		}else{
			$choiceList = $recordtypeobj->getChoiceList($fname,$recid,$idflag);
			@Clist = split(/\,/,$choiceList);
			foreach $val (@Clist) {
				if($fvalue eq $val) { 
						return 1;
				}
			}
			ThrowValidationException("CQCLI_Invalid_FieldValue",$fname);
		}
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_Invalid_FieldValidJava");
		}
		else{
			ThrowException("CQCLI_Invalid_FieldValidPerl",$@);
		}
	}finally{};
}

sub GetFieldValueInterActively
{
	my $fieldname = shift;
	my $rtype = shift;
	my $recid = shift;
	my $idflag = shift;
	my ($promptmsg,$repromptmsg,$cnt,$val)=(0)*4;
	my ($choiceList, $fieldvalue,$i,$isChiceList) =(0)*4;
	my @Clist = ();
	my $isExist = undef;
	

	try {
		$recordtypeobj = GetRecordTypeInfoObj($rtype);

		$isChiceList = $recordtypeobj->isChoiceList($fieldname,$recid,$idflag);

		if($isChiceList) {
			$promptmsg = GetLocalizedMessage('CQCLI_Prompt_FieldValueClist',$fieldname);
			$repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_FieldValueClist',$fieldname);
			$choiceList = $recordtypeobj->getChoiceList($fieldname,$recid,$idflag);
			@Clist = split(/\,/,$choiceList);
			if($choiceList =~ /\,/ ) {
				$choiceList =~s/\,/\n/g;
			}
			$choiceList = $choiceList."\n\n";
		}else{
			$promptmsg = GetLocalizedMessage('CQCLI_Prompt_FieldValue',$fieldname);
			$repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_FieldValue',$fieldname);
		}

		$i = 0;
		$cnt = GetPromptCount();
		while($i<$cnt)
		{
			CLIPrintOutput($promptmsg);
			$fieldvalue=<STDIN>;
			chomp($fieldvalue);
			$fieldvalue=lc cqclitrim($fieldvalue);
			if($isChiceList) {
				if(length $fieldvalue && $fieldvalue =~ /^\?/){
					CLIPrintOutput($choiceList);
					redo;
				}elsif(length $fieldvalue){
					foreach $val (@Clist) {
						my $tmp = lc $val;
						if($fieldvalue eq lc $val) { 
							return $fieldvalue;
						}
					}
					$promptmsg=$repromptmsg;
				}else {
					$promptmsg=$repromptmsg;
				}

			}else{
				return $fieldvalue;
			}

			$i++;
		}
		ThrowValidationException("CQCLI_Invalid_FieldValue",$fieldname);
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception") || caught("java.lang.Throwable")){
			ThrowJavaException($@,"CQCLI_Invalid_FieldValidJava");
		}
		else{
			ThrowException("CQCLI_Invalid_FieldValidPerl",$@);
		}
	}finally{};

}
sub GetAllFieldsInterActively
{
	my $rtype = shift;
	my $recid = shift;
	my $idtype = shift;
	my ($allfields_desc,$recordtypeobj,$allfields) =(0)*3;
	my ($promptmsg,$repromptmsg) = (0)*2;
	my ($fieldname,$expectedfield,$i,$cnt) = (0)*2;
	my %FieldHash = ();

	try {
		$recordtypeobj = GetRecordTypeInfoObj($rtype);

		$allfields_desc = $recordtypeobj->getFieldDefintions();

		$allfields = $allfields_desc;
		$allfields =~ s/=[^\,]+\,?/\n/g;
		$allfields = $allfields."\n\n";
		$promptmsg = GetLocalizedMessage('CQCLI_Prompt_FieldName');
		$repromptmsg = GetLocalizedMessage('CQCLI_RePrompt_FieldName');
	
		$cnt = GetPromptCount();
		$i = 0;
		while($i<$cnt)
		{	
			CLIPrintOutput($promptmsg);
			$fieldname=<STDIN>;
			chomp($fieldname);
			#RATLC01354985  - Resolution field name entered through the interactive mode always warns of multiple matches
			$fieldname=cqclitrim($fieldname);
			if(length $fieldname && $fieldname =~ /^\?/){
				CLIPrintOutput($allfields);
				redo;
			}elsif(length $fieldname) {
				$expectedfield = $recordtypeobj->isValidFieldName($fieldname);
				#ask prashanth for this
				if($expectedfield) {
					$expectedfield =~ s/^\,//g;
				}
				if(!($expectedfield)) {
					$promptmsg=$repromptmsg;
				}elsif($expectedfield =~ /\,/){
					$promptmsg = GetLocalizedMessage('CQCLI_RePrompt_FieldAmbiguity',$fieldname,$expectedfield);
				}else{
					$FieldHash{$expectedfield} = GetFieldValueInterActively($expectedfield,$rtype,$recid,$idtype);
					if(AskForYN('CQCLI_Prompt_Continue_field'))
					{
						$i=0;
						redo;
					}else {
						return \%FieldHash;
					}
				}
			} else {
				$promptmsg=$repromptmsg;
			}
			$i++;
		}
		ThrowValidationException("CQCLI_Invalid_FieldInput",$rtype);
	}

	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_Invalid_FieldValidJava");
		}
		else{
			ThrowException("CQCLI_Invalid_FieldValidPerl",$@);
		}
	}finally { };	
}

sub GetCommandMode()
{
	
	#0 for api mode
	#1 for non interactive mode
	#2 for interactive
	

}

sub GetPromptCount()
{
	#default count which can be reconfigured
	return 3;
}



sub GetUserName()
{
	CheckPromptModeValidity('CQCLI_Info_Login');
	return(AskForUserName());
}

sub GetPassword()
{
	CheckPromptModeValidity('CQCLI_Info_Login');
	return(AskForPassword());
}

sub GetDBName()
{
	CheckPromptModeValidity('CQCLI_Info_Login');
	return(AskForDBName());
}

sub GetSchemaName()
{
	CheckPromptModeValidity('CQCLI_Info_Login');
	return(AskForConnectionName());
}

sub GetRecordId
{
	my $duplicate = shift;
	my $id = undef;
	CheckPromptModeValidity();
	if($duplicate){
		$id = AskForDuplicateRecordId();
	}else {
		$id = AskForRecordId('1');
	}
	return $id;
}

sub GetDbId()
{
	my $id = undef;
	CheckPromptModeValidity();
	$id = AskForRecordId();
	return $id;
}

sub GetLoginCredentials()
{
	
}

sub GetActionName
{
	my ($id , $type , $idtype) = @_;
	CheckPromptModeValidity();
	my $recordinfo = GetRecordInfoObj();
	my $actionlist = $recordinfo->getLegalActions($id , $type , $idtype);
	$actionlist = cqclitrim($actionlist);
	if($actionlist) {
		return(AskForActionName($actionlist));
	}else {
		ThrowValidationException('CQCLI_Invalid_NoAction');
	}
}

sub GetRequestObject()
{
	my $req = $RESOURCECACHE->{'requestobject'};
	return $req;
}

sub GetResponseObject()
{
	my $res = $RESOURCECACHE->{'responseobject'};
	return $res;
}
sub IsAskMode()
{
	if(IsAPIMode()){
		return 0;
	}elsif( ($CLICmdMode->{'ask'}) ) {
		return 1;
	}

	return 0;
}
sub IsAPIMode()
{
	return $CLICmdMode->{'api'};
}

sub IsInteractiveMode()
{
	return $CLICmdMode->{'interactive'}; 
}

sub SetCQCLIAskMode($)
{
	if(!($CLICmdMode->{'api'})){
		$CLICmdMode->{'ask'} = shift;
	}else {
		$CLICmdMode->{'ask'} = 0;
	}
}

sub SetCQCLIAPIMode($)
{
	$CLICmdMode->{'api'} = shift;
}

sub SetCQCLIInteractiveMode($)
{
	if(!($CLICmdMode->{'api'})){
		$CLICmdMode->{'interactive'} = shift;
	}else {
		$CLICmdMode->{'interactive'} = 0;
	}
}

sub SetRequestObject
{
	$RESOURCECACHE->{'requestobject'} = shift;
	return 1;
}

sub SetResponseObject
{
	$RESOURCECACHE->{'responseobject'} = shift;
	return 1;
}

sub SetSessionScope
{
	$RESOURCECACHE->{'sessionscope'} = shift;
	return 1;
}

sub IsCommandScopeSession()
{
	return $RESOURCECACHE->{'sessionscope'};
}
###########################################################################
#All Private Methods
###########################################################################
sub CheckPromptModeValidity
{
	my $msgid = shift;
	my $replace = shift;
	
	if(IsAPIMode() || !(IsInteractiveMode())) {
		ThrowValidationException($msgid,undef);
		return 1;
	}
	return 0;

}

sub InitCommandMode
{
	$CLICmdMode = { 'api' => undef,
				 'ask' => undef,
				 'interactive' => undef,
				 'promptmode'    => undef,
	};

	$RESOURCECACHE = {
		'controller'	=> undef ,
		'session'		=> undef ,
		'smanager'		=> undef ,
		'rinfo'			=> undef,
		'rtypeinfo'		=> { },
		'sessionscope'	=> undef,
		'responseobject' => undef,
		'requestobject'  => undef,
	};

}
sub InitSessionManager()
{
	$RESOURCECACHE->{'sessionm'} = new cqcli::SessionManager();
}

sub InitRecordObj()
{
	$RESOURCECACHE->{'rtypeinfo'}->{'Defect'} = new cqcli::RecordTypeInfo('Defect');
	$RESOURCECACHE->{'rinfo'} = new cqcli::RecordInfo();
}
# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

############################################################




1;
__END__
